#!/usr/bin/perl
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
#  
#  
# Licensed Materials - Property of IBM 
#  
# (C) COPYRIGHT International Business Machines Corp. 2003 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 

#---------------------------------------------------------------------------
# Module:SetupBootUtils
#
# Description:
#   This module consists of subroutines that would be helpful in setting
# up net boots for various purposes like gettting mac address or for kickstar
# installation of nodes etc. Most of the net boots involve setting up similar
# steps like creating dhcp, setting up pxe files, creating ram disk etc.By
# consolidating the subroutines here, future additions can make use of already 
# existing infrastructure.
# 
#----------------------------------------------------------------------------
# @(#)06   1.17   src/csm/install/SetupBootUtils.pm.perl, setup, csm_rrigcl, rrigclt1fh 4/21/03 07:31:33

package SetupBootUtils;

use strict;
BEGIN { $::csmpm = $ENV{'CSM_PM'} ? $ENV{'CSM_PM'} : '/opt/csm/pm'; }
use lib $::csmpm;
use FindBin qw($Bin);
use lib "$Bin";
use Getopt::Std;
use File::Path;	# Provides mkpath() and rmtree();
use File::Copy; # Provides copy();
use NodeUtils;
use CSMDefs;
use InstallUtils;



# Script to set up Kickstart, dhcp, pxe

#---------------------
# Global Variables
#---------------------
%::BOOT_ATTRIBUTES;
$::BOOT_EXIT=0;
# get the definitions we need for this command
#    in this case get_OSDefs uses the attrs of the management
#    server to identify the correct definitions
%::msOSDefs= InstallUtils->get_OSDefs();


# Directories
$::INSTALLDIR		= $::CSMINSTDIR;
$::TFTPBOOT             ="/tftpboot";
$::PXELINUXCFG          =$::TFTPBOOT   .  "/pxelinux.cfg";
$::OLDGETMACSPXE        =$::PXELINUXCFG  . "/getmacs.pxe";
$::GETMACZ              =$::TFTPBOOT   .  "/csm/getmac.gz";
# Files
$::PXE_INSTALL_TMPL	= "/opt/csm/install/install.pxe";
$::PXE_GETMACS_TMPL	= "/opt/csm/install/getmacs.pxe";
$::PXE_DEFAULT_TMPL	= "/opt/csm/install/default.pxe";
$::FIRSTBOOT_TMPL	= "/opt/csm/install/firstboot.tmpl";

# Defaults
$::DEFAULT_PKGPATH	= "/mnt/cdrom";
$::DEFAULT_KSCFG_TMPL	= "/opt/csm/install/kscfg.tmpl";
my @VALID_ATTRIBUTES	= ( "Netmask", 
			    "Gateway", 
			    "Nameserver", 
			    "Nameservers"); 

my @VALID_BOOT_METHODS	= ( "getmacs", 
			    "kickstart", 
		            "sis",
			    "disk" );
my @VALID_HEX_FILE_OPTIONS = ( "yes", 
			      "no", 
		              "onlyhex" );


#Local Variables
my $bootMethod;
my $hexFileOption="yes";
my @inputNodes;
my @targetNodes;
my @okNodes;
#my %attributes;
my @targetNodes;
my @nodeInfo;
my %nodeHash;
# Subroutines
#---------------------


sub commonNetBootSetup
{
  createBootDirectory();
  createDefaultPxe();
}


sub doFullSetup()
{
  &commonNetBootSetup();
  &setupDHCP();
  &setupPxe();
  &copyKernel();
  &makeRAMDisk();
  &setupPxeHex();
}

sub doFullSetupGenerateNoHex
{
 &commonNetBootSetup();
 &setupDHCP();
 &setupPxe();
 &copyKernel();
 &makeRAMDisk();
  
}

sub doSetupGenerateOnlyHex()
{
  &verifyDHCP();
  &verifyPxe();
  &verifyKernel();
  &verifyRAMDisk();
  &verifyTFTP();
  &verifyOthers();
  &setupPxeHex();
}

sub setupDiskBoot
{
  &setupPxe();
  &setupDHCP();
  &setBootLocation();
}

sub createBootDirectory
{
    my ($rc) = 0;
    #NodeUtils->message('I', 'IMsgSETUP_PXE');

    # Create the /tftpboot/pxelinux.cfg directory.
    mkpath ("$::PXELINUX_CFG_DIR", $::VERBOSE, 0777);

    # This directory needs to have '777' permissions because the node
    # needs to access it via tftp after the install is complete.  The
    # mkpath command seems to have trouble setting 777 permissions, so
    # it must be explicitly done with chmod.
    my $cmd = "$::CHMOD 777 $::PXELINUX_CFG_DIR";
    my $output = NodeUtils->runcmd($cmd,0);
    $rc = $::RUNCMD_RC;
    return $rc;
}

sub createDefaultPxe
{
  my $pxetemplate = $::PXE_DEFAULT_TMPL;
  my $pxefile = "$::PXELINUX_CFG_DIR/default";
  &copyfile($pxetemplate, $pxefile);
}

# Run the createdhcp tool to generate the /etc/dhcpd.conf file for use
# during MAC address collection.
sub setupDHCP
{
 
  if($bootMethod eq 'getmacs' )
  {
     my ($cmd, $output);
     NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','I', 'IMsgGENERATING_DHCP' ); 
     my @nodelist = @targetNodes;
     my $nodelist = join(',', @nodelist);
     my $verbose;
     $verbose = ($::VERBOSE ? "-v" : "");
     if (!($verbose)) { $verbose = ($::VERBOSE ? "-V" : ""); }

     my $attrs = "";
     foreach my $attr (keys %::BOOT_ATTRIBUTES)
     {
	$attrs .=  "$attr=" . $::BOOT_ATTRIBUTES{$attr} . " ";
        #print ("$attr\n");
        #exit;
     }
     my $nodelist_file = InstallUtils->make_node_list_file(\@nodelist);

     if ($nodelist) { $cmd = "$::CREATEDHCP $verbose  -f $nodelist_file $attrs";}
     else           { $cmd = "$::CREATEDHCP $verbose $attrs";}
     $output = NodeUtils->runcmd($cmd,0);
     my $fstatus = $::RUNCMD_RC;
     NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','I', 'IMsgShow_Output', $output) if($fstatus); 

     InstallUtils->close_delete_file($::NODE_LIST_FILE, $nodelist_file);

     return $fstatus;
 }elsif($bootMethod eq 'kickstart' || $bootMethod eq 'sis' )
{
   
  #unless(-e "/etc/dhcpd.conf")
  #{
     my ($cmd, $output);
     #NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
     #          'csminstall','I', 'IMsgGENERATING_DHCP' ); 
     my $verbose;
     $verbose = ($::VERBOSE ? "-v" : "");
     if (!($verbose)) { $verbose = ($::VERBOSE ? "-V" : ""); }

     my $attrs = "";
     foreach my $attr (keys %::BOOT_ATTRIBUTES)
     {
	$attrs .=  "$attr=" . $::BOOT_ATTRIBUTES{$attr} . " ";
        #print ("$attr\n");
        #exit;
     }

     $cmd = "$::CREATEDHCP $verbose $attrs";
     $output = NodeUtils->runcmd($cmd,0);
     my $fstatus = $::RUNCMD_RC;
     NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','I', 'IMsgShow_Output', $output) if($fstatus); 

     #return $fstatus;
  #}
  
  
      # Run updatedhcp to add nodes to the /etc/dhcpd.conf file for use during
      # Kickstart install
      my $nodelist = join(',', @targetNodes);
      my $verbose;
      $verbose = ($::VERBOSE ? "-v" : "");
      if (!($verbose)) { $verbose = ($::VERBOSE ? "-V" : ""); }
      if($bootMethod eq 'kickstart') {
          NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','I', 'IMsgUPDATING_DHCP', $nodelist); 
      }
      else {  #$bootMethod eq 'sis' 
          NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','I', 'IMsgUPDATING_DHCP_SIS', $nodelist); 
      }
      my $nodelist_file = InstallUtils->make_node_list_file(\@targetNodes);
      
      my $cmd = "$::UPDATEDHCP $verbose -n $nodelist";
      #my $cmd = "/bin/echo $nodelist | $::UPDATEDHCP $verbose -f -";
      my $output = NodeUtils->runcmd($cmd,0);
      my $fstatus = $::RUNCMD_RC;
      NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','I', 'IMsgShow_Output', $output) if($fstatus); 
      #remove the nodes for which the update did not work 
      InstallUtils->close_delete_file($::NODE_LIST_FILE, $nodelist_file);
      return $fstatus; # 0 is good  

}
 elsif($bootMethod eq "disk" )
    {
      my $nodelist = join(',', @targetNodes);
      my $verbose;
      $verbose = ($::VERBOSE ? "-v" : "");
      if (!($verbose)) { $verbose = ($::VERBOSE ? "-V" : ""); }
      NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','I', 'IMsgUPDATING_DHCP', $nodelist); 
      my $cmd = "$::UPDATEDHCP $verbose -n $nodelist";
      #for disk
      #If a MAC address exists in the managed node class, create a static DHCP
      # entry . If no MAC address , remove the DHCP range entry for the node.
    }
}

sub setupGetmacsPxeWithoutConsoleSerialDevice
{
  my $pxetemplate = $::PXE_GETMACS_TMPL;
  my ($pxefile) = @_;
  
  open (PXETEMPLATE, "<$pxetemplate")
             || NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH,'csminstall','E2', 'EMsgCANT_READ_FILE', $pxetemplate);

  open (PXEFILE, ">$pxefile")
            || NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall','E2', 'EMsgCANT_WRITE_FILE', $pxefile);

  while (<PXETEMPLATE>)
  {
    s/console=#CONSOLE_DEVICE#,9600//g; # Console Device e.g. ttyS0
    s/SERIAL #CONSOLE_DEVICE_NUMBER# 9600//g; # and number
    print PXEFILE $_;
  }
  close PXETEMPLATE;
  close PXEFILE;
}

sub setupGetmacsPxeWithConsoleSerialDevice
{
  my $pxetemplate = $::PXE_GETMACS_TMPL;
  my ($pxefile, $console_device) = ($_[0], $_[1]);

  $::CONSOLE_DEVICES{$console_device} = 1;# save list of unique devices
  my $console_device_number = $console_device;
  $console_device_number =~ s/\D//g;      # remove all non-digits
  open (PXETEMPLATE, "<$pxetemplate")
            || NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall', 'E2', 'EMsgCANT_READ_FILE', $pxetemplate);

  open (PXEFILE, ">$pxefile")
            || NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall', 'E2', 'EMsgCANT_WRITE_FILE', $pxefile);

  while (<PXETEMPLATE>)
  {    
    # take out the following two entries related to, console serial device
    s/#CONSOLE_DEVICE#/$console_device/g; # Console Device e.g. ttyS0
    s/#CONSOLE_DEVICE_NUMBER#/$console_device_number/g; # and number

    print PXEFILE $_;
  }

  close PXETEMPLATE;
  close PXEFILE;
}

sub isVersion73orAbove
{
  # If the management server is rh7.2 or below, then we need
  # to take the rw option out of the pxegetmacs hex file.

  my ($version73orAbove,$distro,$version,$mod);
  # by default assume it is rh 7.3 or above.
  $version73orAbove = "true";
  # get the distro (i.e.: 7.2, 7.3, 8.0, etc.)
  $distro = InstallUtils->get_DistributionVersion();
  ($version, $mod) = split /\./ , $distro;
  # check if it's less than 7.3:
  if($version <= 7 && $mod <= 2){
    #print "not 7.3 or above...\n";
    $version73orAbove = 0;
  }
  return $version73orAbove;

}


sub setupKickstartPxe
{
    my ($pxefile, $ms_ipaddr, $ksfile, $ksfile_dir, $console_device) = @_;
    my $pxetemplate = $::PXE_INSTALL_TMPL;
    my $ks_level = "ks" . $::DISTRO_VERSION;    # e.g. ks7.3
    open (PXETEMPLATE, "<$pxetemplate")
	|| NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall','E2', 'EMsgCANT_READ_FILE', $pxetemplate);
    open (PXEFILE, ">$pxefile")
	|| NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall', 'E2', 'EMsgCANT_WRITE_FILE', $pxefile);
    if($console_device){
	my $console_device_number = $console_device;
	$console_device_number =~ s/\D//g;      # remove all non-digits
	while (<PXETEMPLATE>)
	{
	    s/#NFS_IP#/$ms_ipaddr/g;         # Management server (NFS server)
	    s/#KS_LEVEL#/$ks_level/g;        # Kickstart Level e.g. ks71
	    s/#KS_DIR#/$ksfile_dir/g;        # Kickstart File Directory
	    s/#KS_FILE#/$ksfile/g;           # Kickstart File <nodeip>-kickstart
	    s/#CONSOLE_DEVICE#/$console_device/g; # Console Device e.g. ttyS0
	    s/#CONSOLE_DEVICE_NUMBER#/$console_device_number/g; # and number
	    print PXEFILE $_;
	}
    }
    else{
	while (<PXETEMPLATE>)
	{
	    s/#NFS_IP#/$ms_ipaddr/g;         # Management server (NFS server)
	    s/#KS_LEVEL#/$ks_level/g;        # Kickstart Level e.g. ks71
	    s/#KS_DIR#/$ksfile_dir/g;        # Kickstart File Directory
	    s/#KS_FILE#/$ksfile/g;           # Kickstart File <nodeip>-kickstart
	    # take out the following two entries related to, 
	    # console serial device
	    s/console=#CONSOLE_DEVICE#,9600//g; # Console Device e.g. ttyS0
	    s/SERIAL #CONSOLE_DEVICE_NUMBER# 9600//g; # and number
	    print PXEFILE $_;
	}
    }
    close PXETEMPLATE;
    close PXEFILE;
}


sub setupSisInstallPxeWithoutConsoleSerialDevice
{
  my ($pxefile) = @_;
  my $pxetemplate = $::PXE_INSTALL_TMPL;
  my $sis_level = "sis" . $::DISTRO_VERSION;	# e.g. sis7.2

  open (PXETEMPLATE, "<$pxetemplate")
   || NodeUtils->messageFromCat('csmInstall.cat', $::MAGMAPPATH, 'csminstall','E2', 'EMsgCANT_READ_FILE', $pxetemplate);
 
  open (PXEFILE, ">$pxefile")
   || NodeUtils->messageFromCat('csmInstall.cat', $::MAGMAPPATH, 'csminstall','E2', 'EMsgCANT_WRITE_FILE', $pxefile);

  while (<PXETEMPLATE>)
  {
    s/#SIS_LEVEL#/$sis_level/g;	     # SIS Level
    # take out the following two entries related to, 
    # console serial device
    s/console=#CONSOLE_DEVICE#,9600//g; # Console Device e.g. ttyS0
    s/SERIAL #CONSOLE_DEVICE_NUMBER# 9600//g; # and number

    print PXEFILE $_;
  }

}

sub setupSisInstallPxeWithConsoleSerialDevice
{

  my ($pxefile, $console_device) = @_;
  my $pxetemplate = $::PXE_INSTALL_TMPL;
  my $sis_level = "sis" . $::DISTRO_VERSION;	# e.g. sis7.2
  my $console_device_number = $console_device;
  $console_device_number =~ s/\D//g;      # remove all non-digits

  open (PXETEMPLATE, "<$pxetemplate")
    || NodeUtils->messageFromCat('csmInstall.cat', $::MAGMAPPATH, 'csminstall','E2', 'EMsgCANT_READ_FILE', $pxetemplate);
  
  open (PXEFILE, ">$pxefile")
    || NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall', 'E2', 'EMsgCANT_WRITE_FILE', $pxefile);
  
  while (<PXETEMPLATE>)
  {
    s/#SIS_LEVEL#/$sis_level/g;	     # SIS Level
    s/#CONSOLE_DEVICE#/$console_device/g; # Console Device e.g. ttyS0
    s/#CONSOLE_DEVICE_NUMBER#/$console_device_number/g; # and number

      print PXEFILE $_;
  }

  close PXETEMPLATE;
  close PXEFILE;

}

sub setupPxe
{
  if($bootMethod eq 'getmacs')
  {
    my $pxetemplate = $::PXE_GETMACS_TMPL;
    my $version73orAbove = &isVersion73orAbove();

    # This must be expanded to process the REAL list of nodes
    foreach my $node (@targetNodes)
    {
       my ($nodename, $nodeip) = NodeUtils->getHost($node);
       my $pxefile = "$::PXELINUX_CFG_DIR/$nodename.getmacs";
       my $console_device = $nodeHash{$nodename}{"ConsoleSerialDevice"};

       # If the ConsoleSerialDevice attribute is set to 'NONE', this is
       # A System that has no console serial device.
       if ($console_device eq 'NONE') 
       {
         &setupGetmacsPxeWithoutConsoleSerialDevice($pxefile);                            
       }
       else
       {
         # If the ConsoleSerialDevice attribute is unset, assume this is
         # a system that was migrated from early CSM 1.2 which hardcoded ttyS1
         if ($console_device eq '') 
         {
           $console_device = "ttyS1";
         }
         &setupGetmacsPxeWithConsoleSerialDevice($pxefile,$console_device);             
       }
	 
       # one last edit, get rid of the read/write option in the kernel
       if (!($version73orAbove || $::DISTRO_NAME =~ /SLES/))
       {
         NodeUtils->runcmd("perl -pi -e \"s/APPEND rw/APPEND/g\" $pxefile");
       }

     } # end of for loop    
  } # end of if
  elsif($bootMethod eq  'kickstart')
  {
      
      # for kickstart
      # Create an install pxe configuration file for each node.
      # The name of the file is /tftpboot/pxelinux.cfg/<nodename>.install.
      
      my $pxetemplate = $::PXE_INSTALL_TMPL;
      my $ks_level = "ks" . $::DISTRO_VERSION;    # e.g. ks7.3
      
      # This must be expanded to process the REAL list of nodes
      foreach my $node (@targetNodes)
      {
        my ($nodename, $nodeip) = NodeUtils->getHost($node);
        my $ksfile_dir;

        # Get the hostname of the management server as known by this node
        #my ($ms_hostname, $ms_ipaddr) = &get_node_management_server($nodename);
        my ($isvr_hostname, $isvr_ip, $dir) = &get_node_isvr($nodename);
	# Kickstart config file directory.
	# Example:  /csminstall/csm/1.2.0/kickstart.RedHat7.3
	if($dir){ #install server is used
	    $ksfile_dir = "$dir/csm/" . $::CSM_VERSION . "/kickstart." . $::DISTRO_NAME . $::DISTRO_VERSION;
	}
	else{
	    $ksfile_dir = "/csminstall/csm/" . $::CSM_VERSION . "/kickstart." . $::DISTRO_NAME . $::DISTRO_VERSION;
	}       
        my $pxefile = "$::PXELINUX_CFG_DIR/$nodename.install";
        my $ksfile = "${nodeip}-kickstart";

        my $console_device = $nodeHash{$nodename}{"ConsoleSerialDevice"};
        # If the ConsoleSerialDevice attribute is set to 'NONE', this is
        # a system that has no console serial device.
        if ($console_device eq 'NONE') 
        { 
	    &setupKickstartPxe($pxefile, $isvr_ip, $ksfile, $ksfile_dir);
	}
        else
	{
          # If the ConsoleSerialDevice attribute is unset, assume this is
	  # a system that was migrated from early CSM 1.2 which hardcoded ttyS1
	  if ($console_device eq '') 
	  {
            $console_device = "ttyS1";
	  }
	  my $console_device_number = $console_device;
	  $console_device_number =~ s/\D//g;      # remove all non-digits
	  &setupKickstartPxe($pxefile, $isvr_ip, $ksfile, $ksfile_dir, $console_device);
	}
        
      }#end of for loop
      &createKickstartFiles();
    }
    elsif($bootMethod eq 'sis')
      {
	# for SIS
	# Create an install pxe configuration file for each node.
        # The name of the file is /tftpboot/pxelinux.cfg/<nodename>.sis.
	my $pxetemplate = $::PXE_INSTALL_TMPL;
	my $sis_level = "sis" . $::DISTRO_VERSION;	# e.g. sis7.2

	# SIS config file directory.  
	# Example:  /csminstall/csm/1.2.0/sis.SLES7.2
	my $sisfile_dir = 
	"/csminstall/csm/" . $::CSM_VERSION . "/sis." . 
	$::DISTRO_NAME . $::DISTRO_VERSION;

	# This must be expanded to process the REAL list of nodes
	foreach my $node (@targetNodes)
	{
	  my ($nodename, $nodeip) = NodeUtils->getHost($node);
	  my $pxefile = "$::PXELINUX_CFG_DIR/$nodename.sis";

	  # Change for console number support
	  my $console_device = $nodeHash{$nodename}{"ConsoleSerialDevice"};
	  # If the ConsoleSerialDevice attribute is set to 'NONE', this is
	  # a system that has no console serial device.
	  if ($console_device eq 'NONE') 
	  {  
            &setupSisInstallPxeWithoutConsoleSerialDevice($pxefile);          	   
	  }
          else
	  {
	    # If the ConsoleSerialDevice attribute is unset, assume this is
	    # a system that was migrated from early CSM 1.2 which hardcoded ttyS1
	    if ($console_device eq '') 
            {
	      $console_device = "ttyS1";
	    }
	    my $console_device_number = $console_device;
	    $console_device_number =~ s/\D//g;      # remove all non-digits
            &setupSisInstallPxeWithConsoleSerialDevice($pxefile,$console_device);   	
	  }
	}#end of for
      } # end of if (sis)
  if($bootMethod eq 'disk')
  {
     my $InDir = "/tftpboot/pxelinux.cfg";
     my @nodeList = @targetNodes;
     my @ips;
     foreach my $node (@nodeList)
     {
       my $ip = NodeUtils->getHost($node);
       push(@ips, $ip);
     }
     my %hexNames = InstallUtils->genHex(@ips);
     foreach(keys %hexNames)
     {
       my $HEX = $hexNames{$_};
       if( -f "$::PXELINUXCFG\/$HEX")
       {
         NodeUtils->runcmd("$::RM -rf $::PXELINUXCFG\/$HEX", 1);
       }
     }
   }# end of if($bootMEthod='disk') 
}

# Create a kickstart configuration file for each node.
sub createKickstartFiles
{
    my $kscfg_tmpl="";
    my @nodeList = @targetNodes;
   
    # Set some variables that will be substituted into the kickstart
    # configuration template file in place of #VARIABLE#.
    my $ks_level = "ks" . $::DISTRO_VERSION;		# e.g. ks7.3

    # Kickstart config file directory.  
    # Example:  /csminstall/csm/1.2.0/kickstart.RedHat7.3
       my $ksfile_dir = 
	"/csminstall/csm/" . $::CSM_VERSION . "/kickstart." . 
	$::DISTRO_NAME . $::DISTRO_VERSION;

    my $nfs_dir = $::REDHAT_TOP;

    # Get the management server's timezone information
    my $clockcfg = "/etc/sysconfig/clock";
    my $zone = `$::GREP "ZONE" $clockcfg`;
    my ($junk, $timezone) = split('=', $zone);
    $timezone =~ s/"//g;
    $timezone =~ s/'//g;

    # Get the management server's timezone --utc flag.
    my $utc = `$::GREP "UTC" $clockcfg`;
    my ($junk, $timezone_utc) = split('=', $utc);
    $timezone_utc =~ s/"//g;
    $timezone_utc =~ s/'//g;
    if (($timezone_utc =~ /yes/i) || ($timezone_utc =~ /true/i))
    {
	$timezone_utc = "--utc";
    }
    else { $timezone_utc = ""; }


    my $nameservers = $::ATTRS{"Nameservers"};

    # Generate a hex representation of each node's ip address.
    my %hex = InstallUtils->genHex(@nodeList);

    # Create a file for each node
    foreach my $node (@nodeList)
    {
         # This is the template that will be filled in.
         # Example:  /opt/csm/install/kscfg.tmpl.RedHat7.3 or could be some file name
         #that was set by csmsetupks using -k flag
        $kscfg_tmpl = $nodeHash{$node}{'InstallTemplate'} ;
	my ($node_hostname, $node_ipaddr) = NodeUtils->getHost($node);
	my $node_hex = $hex{$node_hostname};

	# Get the hostname of the management server as known by this node
	my ($ms_hostname, $ms_ipaddr) = 
		    &get_node_management_server($node_hostname);

	my $ms_shorthost = (split('\.', $ms_hostname))[0];
	(my $dnsdomain = $node) =~ s/^.*?\.//;
	if ($dnsdomain eq $node) 
	{
	    # If the node's hostname has no domain name (short hostname), 
	    # use the domain name of the management server.
	    ($dnsdomain = $ms_hostname) =~ s/^.*?\.//;
	    if ($dnsdomain eq $ms_hostname)
	    {
		$dnsdomain = "";
	    }
	}

	# The NFS Hostname and IP addr are taken from the InstallServer attribute
	# or default to the management server.
	my ($nfs_hostname, $nfs_ipaddr, $dir) = &get_node_isvr($node);
	
	my $csm_mount = "/csminstall/csm";
	# The NFS Directory is taken from the NFSDir attribute (no longer used), or defaults
	# to the location on the management server.
  	my $nfs_dir		= $::REDHAT_TOP;
	if($dir){
	    $nfs_dir =~ s/\/csminstall/$dir/;
	    $csm_mount =~ s/\/csminstall/$dir/;
	}

	my $ksfile = $ksfile_dir . "/" . $node_ipaddr . "-kickstart";

	NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall','V', 'IMsgCREATING_KS_FILE', $ksfile);
	mkpath ("$ksfile_dir", $::VERBOSE, 0755);

	open (KSTEMPLATE, "<$kscfg_tmpl")
	    || NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall', 'E2', 'EMsgCANT_READ_FILE', $kscfg_tmpl);

	open (KSFILE, ">$ksfile")
	    || NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall','E2', 'EMsgCANT_WRITE_FILE', $ksfile);

	# Substitute values in the kickstart configuration template file
	# with variables defined above.
	while (<KSTEMPLATE>)
	{
	    s/#MGMTSVR_HOSTNAME#/$ms_hostname/g; # Management Server hostname
	    s/#MGMTSVR_SHORTHOST#/$ms_shorthost/g; # Management Server shorthost
	    s/#MGMTSVR_IP#/$ms_ipaddr/g;	# Management Server IP Address
	    s/#NODE_HOSTNAME#/$node_hostname/g;	# Node hostname
	    s/#NODE_HEX#/$node_hex/g;		# Node ip address in hex.
	    s/#NFS_HOSTNAME#/$nfs_hostname/g;	# NFS server hostname
	    s/#NFS_IP#/$nfs_ipaddr/g;		# NFS server ip address
	    s/#NFS_DIR#/$nfs_dir/g;		# REDHAT_TOP
	    s/#CSM_MOUNT#/$csm_mount/g;         # CSM mount point
	    s/#TFTP_IP#/$ms_ipaddr/g;		# TFTP Server(Management server)
	    s/#NAMESERVERS#/$nameservers/g;	# Nameservers (comma-separated)
	    s/#DNSDOMAIN#/$dnsdomain/g;		# Node's DNS Domain name
	    s/#CSM_VERSION#/$::CSM_VERSION/g;	# Version of CSM
	    s/#TIMEZONE#/$timezone/g;		# Timezone of MS
	    s/#TIMEZONE_UTC#/$timezone_utc/g;	# Timezone --utc flag of MS

	    print KSFILE $_;
	}
	close(KSTEMPLATE);
	close(KSFILE);
    }
}


#get the ip address, and directory, of the install server for this node
sub get_node_isvr{
    my ($nodename) = @_;
    my $isvr_ip;
    my ($hostname, $dir) = split ':', $nodeHash{$nodename}{'InstallServer'};
    if($hostname){
	($hostname, $isvr_ip) = NodeUtils->getHost($hostname); 
	if(!$dir){
	    $dir = $::SERVER_DIRECTORY;
	}
    }
    else{
	($hostname, $isvr_ip)= &get_node_management_server($nodename);
    }
    
    return($hostname, $isvr_ip, $dir);
}


sub copyKernel
{
  #for getmac
  # Copy the kernel, ramdisk and pxelinux files needed for MAC address collection
  if($bootMethod eq 'getmacs')
  {
    my ($cmd, $rc);

    # Copy the current linux kernel
    mkpath ("$::TFTPBOOT/csm", $::VERBOSE, 0755);
    my $linuzFile = "/boot/vmlinuz";
     # SuSE SLES doesn't have the vmlinuz-<version> file.
    if (( $::DISTRO_NAME =~ /SLES/ ) || ($::DISTRO_NAME =~ /SuSE/)) 
    {
	}
	else {
       chomp(my $lversion = NodeUtils->runcmd("/bin/uname -r")); 
       $linuzFile .= "-$lversion";
    }
    #chomp(my $lversion = NodeUtils->runcmd("/bin/uname -r")); 
    #$linuzFile .= "-$lversion";
    &copyfile ($linuzFile, "$::TFTPBOOT/csm/getmacz");
  }
  elsif($bootMethod eq 'kickstart')
    {
      # FOR KICKSTART
      my ($cmd);
      my ($rc) = 0;
      my ($returncode) = 0;
      my ($output);

      my ($source_kernel) = "$::REDHAT_TOP/images/pxeboot/vmlinuz";
      my ($ks_kernel) = $::TFTPBOOT . "/csm/ks" . $::DISTRO_VERSION . "z";

      mkpath ("$::TFTPBOOT/csm", $::VERBOSE, 0755);
      &copyfile($source_kernel, $ks_kernel);

      return $returncode;
    }
    elsif($bootMethod eq 'sis')
      {
	  &copySISKernel();
      }
}
sub copySISKernel
{
    my ($returncode) = 0;

    my ($source_kernel) = "/usr/share/systemimager/boot/i386/standard/kernel";
    my ($sis_kernel) = $::TFTPBOOT . "/csm/sis" . $::DISTRO_VERSION . "z";

    mkpath ("$::TFTPBOOT/csm", $::VERBOSE, 0755);
    &copyfile($source_kernel, $sis_kernel);

    return $returncode;
}

sub setAttributes
{
     # If netmask is not provided, use the netmask of the management server
    if (!$::BOOT_ATTRIBUTES{"Netmask"})
    {
	$::BOOT_ATTRIBUTES{"Netmask"} = 
		(split('Mask:', `$::IFCONFIG eth0 | $::GREP \'inet addr\'`))[1];
	chomp $::BOOT_ATTRIBUTES{"Netmask"};
    }

    # If Gateway is not provided, use the gateway of the management server
    if (!$::BOOT_ATTRIBUTES{"Gateway"})
    {
	$::BOOT_ATTRIBUTES{"Gateway"} = 
	    (split(' ', `$::NETSTAT -rn | $::GREP 0.0.0.0 | $::TAIL -1`))[1];
	chomp $::BOOT_ATTRIBUTES{"Gateway"};
    }

    # If Nameservers is not provided, use the nameservers of the 
    # management server
    if (!$::BOOT_ATTRIBUTES{"Nameservers"})
    {
	my $nameservers = "";
	my $filename = "/etc/resolv.conf";
	open (RESOLV, "<$filename")
	    || NodeUtils->messageFromcat('csmInstall.cat', $::MSGMAPPATH, 'csminstall','E2', 'EMsgCANT_READ_FILE', $filename);

	while (<RESOLV>)
	{
	    my $line = $_;
	    chomp $line;

	    if ($line =~ /^nameserver/i)
	    {

		$nameservers .= "," if ($nameservers);
		$nameservers .= (split(' ', $line))[1];
	    }
	}
	close (RESOLV);
	$::BOOT_ATTRIBUTES{"Nameservers"} = $nameservers;
	chomp $::BOOT_ATTRIBUTES{"Gateway"};
    }

    #print "Netmask     = " . $::BOOT_ATTRIBUTES{"Netmask"} . "\n";
    #print "Gateway     = " . $::BOOT_ATTRIBUTES{"Gateway"} . "\n";
    #print "Nameservers = " . $::BOOT_ATTRIBUTES{"Nameservers"} . "\n";
}

sub copyfile
{
    my ($src, $dest) = @_;

    NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall', 'V', 'IMsgCOPYING_SRC_DST', $src, $dest);
    copy($src, $dest);
    my $rc = $!;
    ($rc) && NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall', 'E1', 'EMsgNO_FILECOPY_RC', $src, $dest, $rc);

    return $rc;
}



sub makeKickstartRAMDisk
  {
      &prepare_mount();
      # for kickstart
      my ($cmd);
      my ($rc) = 0;
      my ($returncode) = 0;
      my ($output);

      my ($initrd) = "$::REDHAT_TOP/images/pxeboot/initrd-everything.img";
      if (! -f $initrd)
      {
	$initrd = "$::REDHAT_TOP/images/pxeboot/initrd.img";
      }
      my ($ramdisk_basename) = "ks" . $::DISTRO_VERSION;		# e.g. ks7.3
      my ($ks_ramdisk) = $::TFTPBOOT . "/csm/" . $ramdisk_basename . ".gz";

      my $tmpdir = "/tmp/ks" . $::DISTRO_VERSION;  # e.g. /tmp/ks7.3
      if (-d $tmpdir)
      {
        rmtree ($tmpdir, $::VERBOSE, 1);
      }
      mkpath ($tmpdir, $::VERBOSE, 0755);
    
    #-------------------------------------------------
    # Unzip the ramdisk
    #-------------------------------------------------
    my $ramdisk = "$tmpdir/$ramdisk_basename";		# /tmp/ks7.3/ks7.3
    #$cmd = "$::GUNZIP -c $tmpdir/initrd.img > $ramdisk";
    $cmd = "$::GUNZIP -c $initrd > $ramdisk";
    $output = NodeUtils->runcmd($cmd,0);
    $rc = $::RUNCMD_RC;
    NodeUtils->messageFromCat('csmInstall.cat', $::MAGMAPPATH, 'csminstall','I','IMsgShow_Output',$output) if ($rc);
    $returncode = $rc if ($rc > $returncode);

    #-------------------------------------------------
    # Mount the ramdisk to kstmp so it can be modified
    #-------------------------------------------------
    my $kstmp = "$tmpdir/kstmp";			# e.g. /tmp/ks7.3/kstmp
    mkpath ($kstmp, $::VERBOSE, 0755);
    $cmd = "$::MOUNT -o loop $ramdisk $kstmp";
    $output = NodeUtils->runcmd($cmd,0);
    $rc = $::RUNCMD_RC;
    NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall','I','IMsgShow_Output',$output) if ($rc);
    $returncode = $rc if ($rc > $returncode);

    #-------------------------------------------------
    # Copy a terminfo file onto the ramdisk
    #-------------------------------------------------
    mkpath ("$kstmp/etc/terminfo/x", $::VERBOSE, 0755);
    mkpath ("$kstmp/etc/terminfo/v", $::VERBOSE, 0755);
    $cmd = "$::COPY -f /usr/share/terminfo/x/xterm $kstmp/etc/terminfo/v/vt100";
    $output = NodeUtils->runcmd($cmd,0);
    $rc = $::RUNCMD_RC;
    NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall', 'I','IMsgShow_Output',$output) if ($rc);
    $returncode = $rc if ($rc > $returncode);

    #-------------------------------------------------
    # Insert the drivers into the ramdisk
    #-------------------------------------------------

    my $kernel_level = "";
    $::FIND = "/usr/bin/find";

    # Copy the modules.cgz file to a temporary location
    $cmd = "$::COPY -f $kstmp/modules/modules.cgz $tmpdir/modules.cgz";
    $output = NodeUtils->runcmd($cmd,0);

    # Unpack the modules.cgz file
    $cmd = "cd $tmpdir; $::GUNZIP -c modules.cgz | $::CPIO -id";
    $output = NodeUtils->runcmd($cmd,0);

    # Determine the kernel level used in the modules.cgz.
    $cmd = "cd $tmpdir; $::LS -d *BOOT";
    $output = NodeUtils->runcmd($cmd,0);
    chomp $output;
    if (-d "$tmpdir/$output")
    {
        $kernel_level = $output;
    }

    # Define all the modules to insert.  The list depends on the kernel level.
    #my $csm_moduledir	= "/opt/csm/install/drivers/$kernel_level";
    my $csm_moduledir	= "/csminstall/csm/drivers/$kernel_level";
    my @drivers = ("e1000", "bcm5700", "mptscsih", "mptbase");
    my @driver_infofiles = ("e1000", "bcm5700", "mpt");

    my (@driver_list, @pcitable_list, @modinfo_list, @dep_list);

    # Only add the drivers that actually exist to the @driver_list.
    # Some drivers are only needed for certain kernel levels.
    foreach my $driver (@drivers)
    {
        my $driver = "$csm_moduledir/${driver}.o";
        push(@driver_list, $driver)     if (-f $driver);
    }

    # Only add the drivers' info files that actually exist to the lists.
    # Some drivers are only needed for certain kernel levels.
    foreach my $infofile (@driver_infofiles)
    {
        my $pcitable = "$csm_moduledir/${infofile}.pcitable";
        my $modinfo = "$csm_moduledir/${infofile}.module-info";
        my $dep = "$csm_moduledir/${infofile}.dep";

        push(@pcitable_list, $pcitable) if (-f $pcitable);
        push(@modinfo_list, $modinfo)   if (-f $modinfo);
        push(@dep_list, $dep)           if (-f $dep);
    }


    # Insert the modules into the modules.cgz 
    if (($#driver_list + 1) > 0)
    {
        $cmd = "$::COPY -f ". join(' ', @driver_list) ." $tmpdir/$kernel_level";
        $output = NodeUtils->runcmd($cmd,0);
    }

    # Repack the modules.cgz file
    $cmd = "cd $tmpdir; $::FIND $kernel_level -type f -print | " .
           "$::CPIO -o -H crc | " .
           "$::GZIP -c -9 > $tmpdir/modules.cgz";
    $output = NodeUtils->runcmd($cmd,0);

    # Copy modules.cgz back into the ramdisk
    $cmd = "$::COPY -f $tmpdir/modules.cgz $kstmp/modules/modules.cgz";
    $output = NodeUtils->runcmd($cmd,0);

    # Add modules' entries to module-info and pcitable
    if (($#pcitable_list + 1) > 0)
    {
        $cmd = "$::CAT ".join(' ',@pcitable_list)." >> $kstmp/modules/pcitable";
        $output = NodeUtils->runcmd($cmd,0);
    }
    if (($#modinfo_list + 1) > 0)
    {
        $cmd="$::CAT ".join(' ',@modinfo_list)." >> $kstmp/modules/module-info";
        $output = NodeUtils->runcmd($cmd,0);
    }

    if (($#dep_list + 1) > 0)
    {
        $cmd = "$::CAT ". join(' ', @dep_list)." >> $kstmp/modules/modules.dep";
        $output = NodeUtils->runcmd($cmd,0);
    }

    # Copy the drivers into /tmp on the ramdisk to fix a possible bug.
    if (($#driver_list + 1) > 0)
    {
        $cmd = "$::COPY " . join(' ', @driver_list) . " $kstmp/tmp";
        $output = NodeUtils->runcmd($cmd,0);
    }

    # RedHat 7.3 uses the tg3.o driver.  It should be using bcm5700.o
    if ($::DISTRO_VERSION eq "7.3")
    {
	# Modify pcitable to replace all occurences of tg3 with bcm5700
	my $pcitable = "$kstmp/modules/pcitable";
	my $tmp_pcitable = "$tmpdir/pcitable.$$";
	open (PCITABLE, "<$pcitable")
            || NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall','E2', 'EMsgCANT_READ_FILE', $pcitable);
	open (TMP_PCITABLE, ">$tmp_pcitable")
            || NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall','E2', 'EMsgCANT_WRITE_FILE', $pcitable);
	while (<PCITABLE>)
	{
	    s/tg3/bcm5700/g;
	    print TMP_PCITABLE $_;
	}
	close PCITABLE;
	close TMP_PCITABLE;
	NodeUtils->runcmd ("$::COPY $tmp_pcitable $pcitable");
    }


    #-------------------------------------------------
    # Unmount the ramdisk
    #-------------------------------------------------
    $cmd = "$::UNMOUNT $kstmp";
    $output = NodeUtils->runcmd($cmd,0);
    $rc = $::RUNCMD_RC;
    NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall', 'I','IMsgShow_Output',$output) if ($rc);
    $returncode = $rc if ($rc > $returncode);

    #-------------------------------------------------
    # Zip up the ramdisk into <distro_version>.gz (e.g. ks7.3.gz)
    #-------------------------------------------------
    $cmd = "$::GZIP -f $ramdisk";
    $output = NodeUtils->runcmd($cmd,0);
    $rc = $::RUNCMD_RC;
    NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall', 'I','IMsgShow_Output',$output) if ($rc);
    $returncode = $rc if ($rc > $returncode);

    # Copy the new ramdisk to /tftpboot
    &copyfile("$ramdisk.gz", $ks_ramdisk);

    # Clean up the temporary directory
    rmtree ($tmpdir, $::VERBOSE, 1);

    return $returncode; 
  }


sub makeRAMDisk
{  
  if($bootMethod eq 'getmacs')
  {
    # Make the getmac ramdisk by running mkgetmacrd
    my $cmd1;
    NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall','I', 'IMsgCREATING_GETMAC_RAMDISK');
    my $verbose;
    $verbose = ($::VERBOSE ? "-v" : "");
    if (!($verbose)) { $verbose = ($::VERBOSE ? "-V" : ""); }
    $cmd1 = "$::MKGETMACRD $verbose 2>/dev/null";
    print "cmd1\n" if $::VERBOSE;
    system("$cmd1");
    if($? >> 8)
    {
	exit 1;
    }
  }
  elsif($bootMethod eq 'kickstart')
    {
      &makeKickstartRAMDisk();
      &copyDrivers();
    }
    elsif($bootMethod eq 'sis')
      {
	  &makeSISRAMDisk();
      }
}
sub makeSISRAMDisk
{
    my ($returncode) = 0;

    my ($initrd) = "/usr/share/systemimager/boot/i386/standard/initrd.img";
    my ($ramdisk_basename) = "sis" . $::DISTRO_VERSION;	
    my ($sis_ramdisk) = $::TFTPBOOT . "/csm/" . $ramdisk_basename . ".gz";

    &copyfile("$initrd", $sis_ramdisk);

    return $returncode;
}

# Copy the drivers (kernel modules) that will be used to update the system
# after it is installed.  The modules are copied to /lib/modules/<kernel>*
# during the kickstart post-install.
sub copyDrivers
{
    my $srcdir = "/opt/csm/install/drivers";
    my $tgtdir = "/csminstall/csm/drivers";
    mkpath ("$tgtdir", $::VERBOSE, 0755);
    my $cmd = "$::COPY -rp $srcdir/* $tgtdir";
    NodeUtils->runcmd("$cmd",0);
    my $rc = $::RUNCMD_RC;

    return ($rc);
}
sub setupPxeHex
{
 
  if($bootMethod eq 'getmacs' )
  {
    my @ips;
    my %extraHex;
    my $file = "/etc/dhcpd.conf";
    my $lnDir = "/tftpboot/pxelinux.cfg";
    my($hostName, $ip) = NodeUtils->getHost(@targetNodes[0]);
    my $firstFile = $hostName . '.getmacs' ;
    my $lnFile = $lnDir . "/" .  $firstFile ; 
    # get all "range" statements in the dhcpd.conf file:
    chomp(my @out = `$::GREP 'range' $file`); 

    # for each "range" entry, get the ip address and put it
    # into a @ip array:
    for (@out)
    {
      s/^\s+//g;
      my $ip = (split(' ', $_))[1];
      $ip =~ s/;//g;
      NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall', 'V','IMsgShow_Output', $ip);
      push @ips, $ip;
    }

    # get the hex representation of these IP addresses and
    # put them in the extraHex hash table:
    %extraHex = InstallUtils->genHex(@ips);
    
    # make a link with each hex to the getmacs file:    
    for(keys %extraHex)
    {
      my $hex = $extraHex{$_};
      NodeUtils->runcmd("$::COPY -f $lnFile $lnDir/$hex");
    }
  }
  elsif($bootMethod eq 'kickstart' )
    {
      # Change the pxelinux HEX file to link to <node_hostname>.install
      my @nodes = @targetNodes;

      # Generate a hex representation of each node's ip address.
      my %hex = InstallUtils->genHex(@nodes);

      my $mode = "777";
      my $errors = 0;

      foreach my $node (@nodes)
      {
	# Copy /tftpboot/pxelinux.pxe/node_hostname.install to the file named
	# with the hex representation of the node's ip address.
	my $nodehex = $hex{$node};
        # Use different pxe config file name for different install methods
        my $src = "$::PXELINUX_CFG_DIR/${node}.install";
	my $tgt = "$::PXELINUX_CFG_DIR/$nodehex";
	NodeUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH, 'csminstall','V', 'IMsgSETTING_PXELINUX_TO_INSTALL', $node);

	# If we do a symlink, the src file gets overwritten when the node 
	# updates the tgt file via tftp.  Therefore, a copy is used instead.
	#NodeUtils->runcmd("$::LN -sf $src $tgt");
	NodeUtils->runcmd("$::COPY $src $tgt");
	if ($::RUNCMD_RC) 
        {
	  NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','W', 'EMsgCANT_COPY', $src, $tgt);
         $errors++;
	}

	# The file needs to be 777 permissions so that it can be modified
        # via tftp.
	NodeUtils->runcmd("$::CHMOD $mode $tgt");
	if ($::RUNCMD_RC) 
        {
	  NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall', 'W', 'EMsgCANT_CHMOD', $tgt, $mode);
         $errors++;
	}
      } # end of for loop
      return $errors;
   }
   elsif($bootMethod eq 'sis' )
   {
      # Change the pxelinux HEX file to link to <node_hostname>.install
      my @nodes = @targetNodes;

      # Generate a hex representation of each node's ip address.
      my %hex = InstallUtils->genHex(@nodes);

      my $mode = "777";
      my $errors = 0;

      foreach my $node (@nodes)
      {
	# Copy /tftpboot/pxelinux.pxe/node_hostname.install to the file named
	# with the hex representation of the node's ip address.
	my $nodehex = $hex{$node};
        # Use different pxe config file name for different install methods
        my $src = "$::PXELINUX_CFG_DIR/${node}.sis";
	my $tgt = "$::PXELINUX_CFG_DIR/$nodehex";
	NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','V', 'IMsgSETTING_PXELINUX_TO_INSTALL', $node);

	# If we do a symlink, the src file gets overwritten when the node 
	# updates the tgt file via tftp.  Therefore, a copy is used instead.
	#NodeUtils->runcmd("$::LN -sf $src $tgt");
	NodeUtils->runcmd("$::COPY $src $tgt");
	if ($::RUNCMD_RC) 
        {
	  NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','W', 'EMsgCANT_COPY', $src, $tgt);
         $errors++;
	}

	# The file needs to be 777 permissions so that it can be modified
        # via tftp.
	NodeUtils->runcmd("$::CHMOD $mode $tgt");
	if ($::RUNCMD_RC) 
        {
	  NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','W', 'EMsgCANT_CHMOD', $tgt, $mode);
         $errors++;
	}
      } # end of for loop
      return $errors;

   }  
}


sub verifyDHCP
{
  if($bootMethod eq 'getmacs' )
  { 
    my @verifiedNodes = @targetNodes;
    my $num = 0;
    my $file = "/etc/dhcpd.conf";
    my @nodeList;
    # make sure the /etc/dhcpd.conf file exists:
    unless(-f $file)
    {
      NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','E1', 'EMsgDOESNT_EXIST',$file);
    }

    # Make sure that each node that we are going to process
    # has an entry in the /etc/dhcpd.conf file in the dynamic
    # entry section. 

    foreach my $node (@verifiedNodes)
    {
      # get the ip address of the node:
      my(undef,$ipaddr) = NodeUtils->getHost($node);

      # check if it's in the dhcpd.conf file:
      my $out = NodeUtils->runcmd("$::GREP 'range $ipaddr' $file", -1);

      # if it is in the dhcpd.conf file, then put it into the @::nodes.
      # @::nodes is the list that will be processed:
      if($out)
      { 
	$num = 1;;
	push @nodeList, $node;

	# if it's not on the dhcpd.conf file, then we signal an error:
	# this node will not be processed.
      }
      else
      { 
        NodeUtils->messageFromCat('W','EMsgNO_DHCP_ENTRY',$node);
      }
   } # end of for  
   @targetNodes = @nodeList ;
 }
  elsif($bootMethod eq 'kickstart') # || $bootMethod eq 'sis' )
  {
   
    my $rc = InstallUtils->verify_dhcp();
    ($rc) && NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall',"E$rc", 'EMsgCANT_START_DHCP');
    return $rc;
  }
}


sub verifyPxe
{
  if($bootMethod eq 'getmacs' )
  {
    my @nodes = @targetNodes ;

    my @verifiedNodes;
    my $pxefile;
    my $return_value = 0; # assume failure
    my @hasSerialDeviceNodes;
    my $out1=0;
    my $out2=0;

    # check that the pxelinux.cfg dir is defined:
    unless( -d $::PXELINUXCFG ){
        NodeUtils->runcmd("$::MKDIR/$::PXELINUXCFG");
    }
    # don't add the nodes that has no console serial device defined,
    # like console=ttyS0,9600 or console=ttyS1,9600, so getmacs won't process
    # such nodes.
    foreach my $node (@nodes){

        # Find the pxe file for this node
        # If it does not exist, assume we are running with early CSM 1.2 files
        # and try using getmacs.pxe
        my ($nodename, $nodeip) = NodeUtils->getHost($node);
        $pxefile = "$::PXELINUXCFG/$nodename.getmacs";
        unless( -f $pxefile ){
            unless( -f $::OLDGETMACSPXE ){
                NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','E','EMsgFILE_NOT_FOUND', $pxefile );
	        return (0); # failure
	    }
	    $pxefile = $::OLDGETMACSPXE;
        }
	# open the PXEFILE
        open (PXEFILE, "<$pxefile")
            || NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','E2', 'EMsgCANT_READ_FILE', $pxefile);
	# go through and see if it has a serial port. 
        while (<PXEFILE>){
            if( /SERIAL 0 9600/  || /SERIAL 1 9600/ || /SERIAL 2 9600/  || /SERIAL 3 9600/  ){
                 $out1 = 1;
            }        
	}

        if ( $out1){
            # this node has serial device defined
            push @hasSerialDeviceNodes, $node;
        }else{
            #print "No Console serial device defined. $node will not be processed. \n";
	}
        close PXEFILE;
        $out1=0;        
    }
    @nodes = @hasSerialDeviceNodes; 

       
      

    # Use the pxe file for the first node
    # If it does not exist, assume we are running with early CSM 1.2 files
    # and try using getmacs.pxe
    my ($nodename, $nodeip) = NodeUtils->getHost($nodes[0]);
    $::GETMACSPXE = "$::PXELINUXCFG/$nodename.getmacs";
    unless( -f $::GETMACSPXE ){
        unless( -f $::OLDGETMACSPXE ){
            NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','E','EMsgFILE_NOT_FOUND', $::GETMACSPXE );
	    return (0);  # failure
        }
        $::GETMACSPXE = $::OLDGETMACSPXE;
    }


    foreach my $node (@nodes){

        # Find the pxe file for this node
        # If it does not exist, assume we are running with early CSM 1.2 files
        # and try using getmacs.pxe
        ($nodename, $nodeip) = NodeUtils->getHost($node);
        $pxefile = "$::PXELINUXCFG/$nodename.getmacs";
        unless( -f $pxefile ){
            unless( -f $::OLDGETMACSPXE ){
                NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','E','EMsgFILE_NOT_FOUND', $pxefile );
	        return (0); # failure
	    }
	    $pxefile = $::OLDGETMACSPXE;
        }

        # Check if this node's pxe file matches the first node's file
        # If it does, add it to the active node list
	my $out = NodeUtils->runcmd("$::DIFF -Bwq $::GETMACSPXE $pxefile", -1);
 	if($out){ 
        #  need a new error msg!!!!
        #   NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
        #        'csminstall','W','EMsgXXXXX',$pxefile,$::GETMACSPXE,$node)
		print "$pxefile is different from $::GETMACSPXE.  $node will not be processed. \n";
        }else{
	   $return_value = 1;
	   push @verifiedNodes, $node;
        }

    }
    @targetNodes = @verifiedNodes;
    return $return_value;
  }
  elsif($bootMethod eq 'kickstart' || $bootMethod eq 'sis' )
  {
  }

}

sub verifyKernel
{
  if($bootMethod eq 'getmacs' )
  {
    my $getmacs_kernel = "$::TFTPBOOT/csm/getmacz" ;
    unless( -f $getmacs_kernel )
    {
      NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','E', 'EMsgFILE_NOT_FOUND', $getmacs_kernel );
    }
  }
  elsif($bootMethod eq 'kickstart' )
  {
    my $ks_kernel = $::TFTPBOOT . "/csm/ks" . $::DISTRO_VERSION . "z";
    unless( -f $ks_kernel )
    {
      NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','E', 'EMsgFILE_NOT_FOUND', $ks_kernel );
    }
  }elsif($bootMethod eq 'sis' )
  {
    my $sis_kernel = $::TFTPBOOT . "/csm/sis" . $::DISTRO_VERSION . "z";
    unless( -f $sis_kernel )
    {
      NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','E', 'EMsgFILE_NOT_FOUND', $sis_kernel );
    }
  }

}

sub verifyRAMDisk
{
  if($bootMethod eq 'getmacs' )
  {
    unless( -f $::GETMACZ )
    {
      NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','E', 'EMsgFILE_NOT_FOUND', $::GETMACZ);
    }
  }
  elsif($bootMethod eq 'kickstart' )
  {
    my ($ramdisk_basename) = "ks" . $::DISTRO_VERSION;
    my ($ks_ramdisk) = $::TFTPBOOT . "/csm/" . $ramdisk_basename . ".gz";
    unless( -f $ks_ramdisk )
    {
      NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','E', 'EMsgFILE_NOT_FOUND', $ks_ramdisk);
    }
  }
  elsif($bootMethod eq 'sis' )
  {
    my ($ramdisk_basename) = "sis" . $::DISTRO_VERSION;	
    my ($sis_ramdisk) = $::TFTPBOOT . "/csm/" . $ramdisk_basename . ".gz" ;
    unless( -f $sis_ramdisk )
    {
      NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','E', 'EMsgFILE_NOT_FOUND', $sis_ramdisk);
    }
  }
  
}
sub verifyTFTP
{
  my $rc =0;
  if( $bootMethod eq 'getmacs' || $bootMethod eq 'kickstart' || $bootMethod eq 'sis' )
  {
    $rc = InstallUtils->verify_tftp();
    ($rc) && NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall',"E$rc", 'EMsgCANT_START_TFTP');
    return $rc;
  }
}

sub verifyOthers
{
  if($bootMethod eq 'getmacs')
  {
  }
  elsif($bootMethod eq 'kickstart' || $bootMethod eq 'sis' )
  {
     #&verify_systemimager();
  }

}


# Verify the systemimager daemon to ensure the rsyncd is running
sub verify_systemimager
{
    my ($InstallMethod);
    &loadNodeAttributes();
	# Find whether there are some nodes use "sis" InstallMethod
    foreach my $node (@targetNodes)
    {
      $InstallMethod = $nodeHash{$node}{'InstallMethod'};
       if ( $InstallMethod =~ "sis" )
        {
       	  my $cmd = "/etc/init.d/systemimager start";
	  my $rc = NodeUtils->runcmd($cmd, 0);
	  return $::RUNCMD_RC;
        }
    }
    
	return 0;
}

sub setBootLocation
{
  #TBD
}
# Get the hostname of the management server for a node.
#
# Since the nodes might need to contact the management server through a
# different address than its primary hostname, we cannot just use the output
# of the hostname command here.
#
# If we cannot contact the node then return the output of the hostname command
# as a backup.
#
sub get_node_management_server
{
    my ($nodename) = @_;

    my ($ms_hostname, $ms_ipaddr);
    $ms_hostname = $nodeHash{$nodename}{"ManagementServer"};
    if ($ms_hostname)
    {
	($ms_hostname, $ms_ipaddr) = NodeUtils->getHost($ms_hostname);
    }
    else
    {
	($ms_hostname, $ms_ipaddr) = 
			    InstallUtils->get_management_server($nodename);
    }

    # If the management server cannot be resolved, just use the output
    # of the hostname command on the management server.  This is done because
    # we require some management server hostname, even if the node cannot
    # be resolved.
    if (! $ms_hostname)
    {
	($ms_hostname, $ms_ipaddr) = &get_management_server_by_hostname();
    }

    return ($ms_hostname, $ms_ipaddr);
}

sub get_management_server_by_hostname
{
    my $h = `hostname`;
    chomp $h;
    my ($hostname, $ipaddr) = NodeUtils->getHost($h);
    chomp $hostname;
    chomp $ipaddr;
    return ($hostname, $ipaddr);
}

# Get the hostname of the management server.
#
# Since the nodes might need to contact the management server through a
# different address than its primary hostname, we cannot just use the output
# of the hostname command here.
#
# On Linux, the ping command outputs a line like the following which is
# parsed to find the originating hostname (m.m.m.m = management server IP addr).
#   PING node_hostname (n.n.n.n) from m.m.m.m : 56(84) bytes of data.
#
# On AIX, the traceroute command outputs a line like the following which is
# parsed to find the originating hostname (m.m.m.m = management server IP addr).
#   source should be m.m.m.m
# (This is not implemented for AIX yet!!!)
#
# If we cannot contact the node via ping (for Linux) or traceroute (for AIX),
# just return nothing.  This indicates that there is no way to get to the node.
#
sub get_management_server
{
    my ($node_hostname) = @_;
    my $ipaddr   = "";
    my $ping_searchstring1 = "\^PING ";
    my $ping_searchstring2 = " from ";
    my $cmd = "$::PING -c1 $node_hostname 2>/dev/null";
    NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','V', 'IMsgCMD', $cmd);
    open (CMD, "$cmd |")
	|| NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
               'csminstall','W', 'EMsgCANT_RUN_CMD', $cmd);
    while (<CMD>)
    {
	# Find the correct line in the ping output
	if (/$ping_searchstring1/)
	{
	    my $line = $_;
	    # Split the line around the "from".
	    my ($before, $after) = split ($ping_searchstring2, $line);
	    # Find the first word after the "from".
	    ($ipaddr) = (split(' ', $after))[0];
	}
    }

    if (!$ipaddr)
    {
	my $ms_hostname;
	($ms_hostname, $ipaddr) = &get_management_server_by_hostname;
    }

    return NodeUtils->getHost($ipaddr);
}

sub getCommonAttributes
{
# Get the attributes that are common across all nodes.
($::DISTRO_NAME,
 $::DISTRO_VERSION, 
 $::ARCH, 
 $::CSM_VERSION) = InstallUtils->get_common_attrs();
}


#prepare_mount sub-routine will check if the director/tmp/... is mounted.
#If mounted, then it will unmount.
sub prepare_mount
{
   &getCommonAttributes();
   my ($cmd, @my_result, $cnt);
   my $tmpdir="/tmp/ks" . $::DISTRO_VERSION;  # e.g. /tmp/ks7.3
   my $check_val=$tmpdir."/kstmp";
   $cmd="$::MOUNT | grep $check_val";
   @my_result=NodeUtils->runcmd($cmd,-1);
   chomp (@my_result);
   $cnt=@my_result;
   if ($cnt > 0)
   {
     my $i;
     for($i=1; $i<=$cnt;$i++)
     {
        $cmd = "$::UNMOUNT $check_val";
        NodeUtils->runcmd($cmd);
     }
   }
}

sub loadNodeAttributes
{
   my @nodeList = @inputNodes;
    # Process the various ways of inputting the list of nodes.  
    my ($ref_DestNode, $ref_lsnode_info, $ref_DestNodeHash) 
		= InstallUtils->get_target_nodes(\@nodeList);
    @targetNodes = @$ref_DestNode;
    @nodeInfo = @$ref_lsnode_info;
    %nodeHash = %$ref_DestNodeHash;
    #@::NODELIST = @$ref_DestNode;
    #@::LSNODE_INFO = @$ref_lsnode_info;
    %::NODEHASH = %$ref_DestNodeHash;
    if(!@targetNodes)
    {
	#NodeUtils->messageFromCat('csmInstall.cat',$::MSGMAPPATH,
       #        'csminstall','E1', 'EMsgINSTALLNODE_NO_TARGET');
    }

} 

sub setupBoot
{
  shift(@_) ;
  my($boot, $hexOption, @nodeList) = @_ ;
  my @commonBootMethods = ("getmacs", "kickstart", "sis");
  my $rc=0;
  $bootMethod = $boot ;
  # print("BOOT_METHOD:,$bootMethod\n");

  $hexFileOption = $hexOption;
  #print("HEX OPTION:,$hexFileOption\n");

  @inputNodes = @nodeList ; #("c5bn03.ppd.pok.ibm.com");
  #print("NODE LIST:@nodeList\n");
  if(! (@inputNodes))
  {
    #print("No Nodes to process\n");
    return (());
  }

  &setAttributes();
  &loadNodeAttributes();
  &getCommonAttributes();

  #Disk boot setup
  if($bootMethod eq "disk" )
  {
    &setupDiskBoot();
  }

  foreach my $bootMethod (@commonBootMethods)
  {
    if($bootMethod =~ /\b$bootMethod\b/i )
    {
       if($hexFileOption eq "yes"  )
       {
         $::BOOT_EXIT = &doFullSetup();
         return (@targetNodes);
       }
       elsif($hexFileOption eq "no"  )
       {
         #print("Before doFullSetupGenerateNoHex...\n");
         $::BOOT_EXIT = &doFullSetupGenerateNoHex();
         return (@targetNodes);
       }
       elsif($hexFileOption eq "onlyhex"  )
       {
	  #print(" before doSetupgenerateOnlyhex..\n");
           $::BOOT_EXIT = &doSetupGenerateOnlyHex();
           return (@targetNodes);
       }
     }
  }
  return (@targetNodes);
}
1;














